/****************************************************************************
 *
 * Copy right:   Qx.CJ
 * File name:    MainTask.c
 * Description:  主要负责：1.工作模式转换2.定时器定时3.定时保存运行数据4.异常状态重启
 * History:      2021-07-05
 * Version:      V3.0
 ****************************************************************************/
#include "MainTask.h"

static StaticTask_t           gProcess_Main_Task_t;
static UINT8                  gProcess_Main_TaskStack[PROC_MAIN_TASK_STACK_SIZE];
static osThreadId_t           MainTaskId = NULL;
static process_Main 		    gProcess_Main_Task;
#define PROC_MAIN_STATE_SWITCH(a)  (gProcess_Main_Task = a)

//函数声明区
static void appBeforeHib(void *pdata, slpManLpState state);
static void appAfterHib(void *pdata, slpManLpState state);
static void appBeforeSlp1(void *pdata, slpManLpState state);
static void appAfterSlp1(void *pdata, slpManLpState state);
static void appBeforeSlp2(void *pdata, slpManLpState state);
static void appAfterSlp2(void *pdata, slpManLpState state);
static void montior_timer_callback(TimerHandle_t xTimer);
static void work_timer_callback(TimerHandle_t xTimer);

static void setDefaultAppDataValue(void);
static void appLoadConfig(void);
static void appSaveConfig(void);
static void appGetNVMSavedData(void);
static void appSaveNVMData(void);
static void setDefaultAppDataInfo(void);
static void LoadAppDataInfo(void);
static void appSaveDataInfo(void);
//主线程任务区
static void MainTask(void* arg)
{
    /***GsensorInit***/
    GsensorI2CHandler(GsensorI2CCallback);
    GsensorInit();
    /***calibrate the ADC Collect***/
    UINT32  param = 0;
    AdcSendReq(ADC_REQ_BITMAP_VBAT_CALI,&param,1,ADC_GET_RESULT_TIMOUT);						
	AdcSendReq(ADC_REQ_BITMAP_INRES_CALI,&param,1,ADC_GET_RESULT_TIMOUT);
    /***Sleep***/
    UINT32 inParam = 0xAABBCCDD;
    slpManSetPmuSleepMode(true,SLP_HIB_STATE,false);
    UINT8 MainSlpHandler=0xff;
    slpManApplyPlatVoteHandle("MainSlp",&MainSlpHandler);
    slpManPlatVoteDisableSleep(MainSlpHandler, SLP_SLP2_STATE);
    slpManRegisterUsrdefinedBackupCb(appBeforeHib,&inParam,SLPMAN_HIBERNATE_STATE);
    slpManRegisterUsrdefinedRestoreCb(appAfterHib,NULL,SLPMAN_HIBERNATE_STATE);
    slpManRegisterUsrdefinedBackupCb(appBeforeSlp1,NULL,SLPMAN_SLEEP1_STATE);
    slpManRegisterUsrdefinedRestoreCb(appAfterSlp1,NULL,SLPMAN_SLEEP1_STATE);
    slpManRegisterUsrdefinedBackupCb(appBeforeSlp2,NULL,SLPMAN_SLEEP2_STATE);
    slpManRegisterUsrdefinedRestoreCb(appAfterSlp2,NULL,SLPMAN_SLEEP2_STATE);
    slpManSlpState_t slpstate = slpManGetLastSlpState(); 

    TimerHandle_t montior_timer = NULL;
    TimerHandle_t work_timer = NULL;
    slpManWakeSrc_e Wakeup_source;
    Wakeup_source =  slpManGetWakeupSrc();//获取唤醒源
    #ifdef USING_PRINTF1
        printf("Wakeup_source:%d \n",Wakeup_source);
    #endif
    #ifdef DEBUGLOG
    Debug_printf("Wkp-%d \n",Wakeup_source);
    #endif

    montior_timer = xTimerCreate("montior_timer", 100 / portTICK_RATE_MS, pdTRUE, NULL, montior_timer_callback);
    work_timer = xTimerCreate("work_timer", AppNVMData.wakeupWorkTime*60*1000 / portTICK_RATE_MS, pdTRUE, NULL, work_timer_callback);
	NetSocDisplay(LED_SOC_0,LED_TURN_OFF);
	NetSocDisplay(LED_SOC_1,LED_TURN_OFF);
	NetSocDisplay(LED_SOC_2,LED_TURN_OFF);
	NetSocDisplay(LED_SOC_3,LED_TURN_OFF);
	FaultDisplay(LED_TURN_OFF);
    PROC_MAIN_STATE_SWITCH(PROCESS_STATE_IDLE);
    while(true)
    {
        switch(gProcess_Main_Task)
        {
            case PROCESS_STATE_IDLE:
            {
                appGetNVMSavedData();
                LoadAppDataInfo();
                TimeCounter = 0;
                xTimerStart(montior_timer, 0);
                xTimerStart(work_timer, 0);
                PROC_MAIN_STATE_SWITCH(PROCESS_STATE_WORK);
                break;
            }
            case PROCESS_STATE_WORK:
            {
                /***if Tcp connect error 5 times then save data and restart sys***/
                if(TcpErrorcount>=5)
                {
                    AppDataInfo.appDataModify = true;
                    AppDataInfo.SysReStart = AppDataInfo.SysReStart +1;
                    if(AppDataInfo.SysReStart>=0xfffe)
                    {
                        AppDataInfo.SysReStart = 0;
                    }
                    appSaveNVMData();
                    appSaveDataInfo();
                    appSetCFUN(0);
                    osDelay(1000);
                    EC_SystemReset();
                }
                
                osDelay(1000);
                break;
            }
            case PROCESS_STATE_SLEEP:
            {
                if(NB_Fota_update_flag)
                {
                    appSaveNVMData();
                    appSaveDataInfo();
                    appSetCFUN(0);
                    osDelay(1000);
                    EC_SystemReset();
                }
                break;
            }
            default:
            {
                PROC_MAIN_STATE_SWITCH(PROCESS_STATE_IDLE);
                break;
            }
        }
    }
}
void MainTaskInit(void *arg)
{
    #ifndef USING_PRINTF
    if(BSP_GetPlatConfigItemValue(PLAT_CONFIG_ITEM_LOG_CONTROL) != 0)
    {
        HAL_UART_RecvFlowControl(false);
    }
    #endif
    osThreadAttr_t task_attr;
    memset(&task_attr,0,sizeof(task_attr));
    memset(gProcess_Main_TaskStack, 0xA5, PROC_MAIN_TASK_STACK_SIZE);
    task_attr.name = "Main_Task";
    task_attr.stack_mem = gProcess_Main_TaskStack;
    task_attr.stack_size = PROC_MAIN_TASK_STACK_SIZE;
    task_attr.priority = osPriorityNormal;
    task_attr.cb_mem = &gProcess_Main_Task_t;
    task_attr.cb_size = sizeof(StaticTask_t);
    MainTaskId = osThreadNew(MainTask, NULL, &task_attr);

}
void MainTaskDeInit(void *arg)
{
    osThreadTerminate(MainTaskId);
    MainTaskId = NULL;
}
//定时器回调函数区
void montior_timer_callback(TimerHandle_t xTimer)
{
    TimeCounter++;
    if(TimeCounter>0xfffffffd)
    {
        TimeCounter=0;
    }
}
void work_timer_callback(TimerHandle_t xTimer)
{
    #ifdef USING_PRINTF
        printf("Hello work Timer!\n");
    #endif
}

//睡眠进出函数区
static void appBeforeHib(void *pdata, slpManLpState state)
{
    //uint32_t *p_param = (uint32_t *)pdata;

    #ifdef USING_PRINTF
        printf("Before Hibernate:%d \n",state);
    #endif
	slpManAONIOLatchEn(AonIOLatch_Enable);

}
static void appAfterHib(void *pdata, slpManLpState state)
{
    #ifdef USING_PRINTF
        printf("Try Hibernate Failed:%d \n",state);
    #endif
}
static void appBeforeSlp1(void *pdata, slpManLpState state)
{
    #ifdef USING_PRINTF
        printf("Before Sleep1:%d \n",state);
    #endif
	slpManAONIOLatchEn(AonIOLatch_Enable);
}
static void appAfterSlp1(void *pdata, slpManLpState state)
{
    #ifdef USING_PRINTF
        printf("After Sleep1:%d \n",state);
    #endif
}
static void appBeforeSlp2(void *pdata, slpManLpState state)
{
    #ifdef USING_PRINTF
        printf("before sleep2:%d \n",state);
    #endif
	slpManAONIOLatchEn(AonIOLatch_Enable);
}
static void appAfterSlp2(void *pdata, slpManLpState state)
{
    #ifdef USING_PRINTF
        printf("sleep2 failed:%d \n",state);
    #endif
}


static void appGetNVMSavedData(void)
{
    appLoadConfig();
}

void appSaveNVMData(void)
{
	appSaveConfig();
}

static void appSaveConfig(void)
{
    OSAFILE fp = PNULL;
    UINT32  writeCount = 0;
    AppConfigHeader    AppConfigHr;   //4 bytes

    /*
     * open the NVM file
    */
    fp = OsaFopen(APP_CONFIG_FILE_NAME, "wb");   //read & write
    if(OsaFseek(fp, 0, SEEK_SET) != 0)
	{
        #ifdef USING_PRINTF
				printf("Seek file failed [%d] \r\n",__LINE__);
	    #endif
	    OsaFclose(fp);
        return;
	}
    if (fp == PNULL)
    {
	   #ifdef USING_PRINTF
			printf(" NVM, can't open/create NVM: 'qxappconfig.nvm', save NVM failed\n");
		#endif
        return;
    }

    /*
     * write the header
    */
    AppConfigHr.fileBodySize   = sizeof(AppNVMData);
    AppConfigHr.version	= 	APP_CONFIG_FILE_LATEST_VERSION;    
    AppConfigHr.checkSum       = OsaCalcCrcValue((UINT8 *)&AppNVMData, sizeof(AppNVMData));

    writeCount = OsaFwrite(&AppConfigHr, sizeof(AppConfigHeader), 1, fp);
    if (writeCount != 1)
    {
        //ECOMM_TRACE(UNILOG_PLA_MIDWARE, AppConfigSave_2, P_ERROR, 0,
        //           " NVM: 'qxappconfig.nvm', write the file header failed");
	#ifdef USING_PRINTF
				printf(" NVM: 'qxappconfig.nvm', write the file header failed\n");
	#endif

        OsaFclose(fp);
        return;
    }

    /*
     * write the file body
    */
    AppNVMData.appDataModify = FALSE;
    writeCount = OsaFwrite(&AppNVMData, sizeof(AppNVMData), 1, fp);
    if (writeCount != 1)
    {
        //ECOMM_TRACE(UNILOG_PLA_MIDWARE, AppConfigSave_3, P_ERROR, 0,
        //           " NVM: 'qxappconfig.nvm', write the file body failed");
        #ifdef USING_PRINTF
					printf(" NVM: 'qxappconfig.nvm', write the file body failed\n");
		#endif
    }
    OsaFclose(fp);
    return;
}


void appLoadConfig(void)
{
    OSAFILE fp = PNULL;
    UINT32  readCount = 0;
    AppConfigHeader    AppConfigHr;   //4 bytes
    UINT8   crcCheck = 0;
    void    *pReadAppConfig = (void *)&AppNVMData;
    /*
     * open the NVM file
    */
    fp = OsaFopen(APP_CONFIG_FILE_NAME, "rb");   //read only
    
    if (fp == PNULL)
    {
	#ifdef USING_PRINTF
		printf(" NVM, can't open NVM: 'qxappConfig.nvm', use the defult value\n");
	#endif
        setDefaultAppDataValue();
        appSaveConfig();
        return;
    }
    /*
     * read the file header
    */
    readCount = OsaFread(&AppConfigHr, sizeof(AppConfigHeader), 1, fp);
    UINT8 readtimes=0;
    while (readCount != 1 && readtimes<=5 )
    {
        //ECOMM_TRACE(UNILOG_PLA_MIDWARE, AppLoadNvmConfig_header_e_1, P_ERROR, 1,
        //            "NVM: 'qxappconfig.nvm', can't read header, return: %d, use the defult value", readCount);
        #ifdef USING_PRINTF
                    printf("NVM: 'qxappconfig.nvm', can't read header, return: %d, use the defult value \n",readtimes);
        #endif
        readtimes++;
        readCount = OsaFread(&AppConfigHr, sizeof(AppConfigHeader), 1, fp);
        osDelay(10);
    }
    if(readtimes>5)
    {
        OsaFclose(fp);
        setDefaultAppDataValue();
        appSaveConfig();
        return;
    }
    if (AppConfigHr.version != APP_CONFIG_FILE_LATEST_VERSION)
    {
        //ECOMM_TRACE(UNILOG_PLA_MIDWARE, AppLoadNvmConfig_ver_w_1, P_ERROR, 2,
        //            "NVM: 'qxappconfig.nvm', version: %d not latest: %d, try to adapt",
        //            AppConfigHr.version, MID_WARE_NVM_FILE_LATEST_VERSION);

		#ifdef USING_PRINTF
				printf("NVM: 'qxappconfig.nvm', version: %d not latest: %d, try to adapt\n",
						AppConfigHr.version, MID_WARE_NVM_FILE_LATEST_VERSION);
		#endif
        if (AppConfigHr.fileBodySize > 1024)   //in fact this NVM size should limited in 1KB
        {
            //ECOMM_TRACE(UNILOG_PLA_MIDWARE, AppLoadNvmConfig_ver_e_1, P_ERROR, 2,
            //            "NVM: 'qxappconfig.nvm', version: %d, NVM body size: %d > 1024, not right, use the default value",
             //           AppConfigHr.version, AppConfigHr.fileBodySize);
			#ifdef USING_PRINTF
				printf("NVM: 'qxappconfig.nvm', version: %d, NVM body size: %d > 1024, not right, use the default value",
                       AppConfigHr.version, AppConfigHr.fileBodySize);
			#endif
            OsaFclose(fp);
            setDefaultAppDataValue();
            appSaveConfig();
            return;
        }

        /*
         * As need to do adaption, can't read the old NVM into "mwNvmConfig", here we allocate a new buffer to store it
        */
        pReadAppConfig  = OsaAllocZeroMemory(AppConfigHr.fileBodySize);

        if (pReadAppConfig == PNULL)
        {
            //ECOMM_TRACE(UNILOG_PLA_MIDWARE, AppLoadNvmConfig_buff_e_1, P_ERROR, 2,
            //            "NVM: 'qxappconfig.nvm', version: %d, can not allo memsize: %d, use the default value",
            //            AppConfigHr.version, AppConfigHr.fileBodySize);
			#ifdef USING_PRINTF
				printf("NVM: 'qxappconfig.nvm', version: %d, can not allo memsize: %d, use the default value",
                        AppConfigHr.version, AppConfigHr.fileBodySize);
			#endif
            OsaFclose(fp);
            setDefaultAppDataValue();
            appSaveConfig();

            return;
        }		
        //needAdjust = TRUE;
    }
    else if (AppConfigHr.fileBodySize != sizeof(AppNVMData))  //file version is the same, but NVM file size not right
    {
        //ECOMM_TRACE(UNILOG_PLA_MIDWARE, AppLoadNvmConfig_2, P_ERROR, 2,
        //            "MW NVM: 'midwareconfig.nvm', file body size not right: (%u/%u), use the defult value",
        //            AppConfigHr.fileBodySize, sizeof(AppNVMData));
		#ifdef USING_PRINTF
				printf("MW NVM: 'midwareconfig.nvm', file body size not right: (%u/%u), use the defult value",
        		            AppConfigHr.fileBodySize, sizeof(AppNVMData));
		#endif
		
        OsaFclose(fp);

        setDefaultAppDataValue();
        appSaveConfig();

        return;
    }

    /*
     * read the file body
    */
    
    readCount = OsaFread(pReadAppConfig, AppConfigHr.fileBodySize, 1, fp);
    crcCheck = OsaCalcCrcValue((UINT8 *)pReadAppConfig, sizeof(AppNVMData));
	readtimes = 0;
    //while (readtimes<=5 && crcCheck != AppConfigHr.checkSum)
    while (readtimes<=5 && readCount != 1)
    {
        readtimes++;
        readCount = OsaFread(pReadAppConfig, AppConfigHr.fileBodySize, 1, fp);
        crcCheck = OsaCalcCrcValue((UINT8 *)pReadAppConfig, sizeof(AppNVMData));
    }
    if (readtimes>5)
    {
        //ECOMM_TRACE(UNILOG_PLA_MIDWARE, AppLoadNvmConfig_3, P_ERROR, 2,
        //            "NVM: 'qxappconfig.nvm', can't read body, or body not right, (%u/%u), use the defult value",
        //            crcCheck, AppConfigHr.checkSum);
		#ifdef USING_PRINTF
			printf(" NVM: 'qxappconfig.nvm', can't read body, or body not right, (%u/%u), use the defult value\n ",
						crcCheck, AppConfigHr.checkSum);
		#endif
        OsaFclose(fp);
        setDefaultAppDataValue();
        appSaveConfig();
        /*
        if (needAdjust)
        {
            OsaFreeMemory(&pReadAppConfig);
        }
        */
        return;
    }

    #if 0
        if (needAdjust)
        {
            OsaFclose(fp);
            setDefaultAppDataValue();
            appSaveConfig();

            /* free memory */
            OsaFreeMemory(&pReadAppConfig);
        }
    #endif
    #if 0 
        int i=0;

        printf("AppConfigHeader:\n");
        printf("%d,%d,%d\n",AppConfigHr.fileBodySize,AppConfigHr.version,AppConfigHr.checkSum);
        printf("AppConfigFileBody:\n");
        printf("%d,%d,%d\n",AppNVMData.chargEndWorkTime,AppNVMData.wakeupWorkTime,AppNVMData.sleepTime);
        printf("\n");
    #endif
    OsaFclose(fp);
    return;
}


//if the AppNVMData.version(read form nvm) != APP_CONFIG_FILE_LATEST_VERSION, it will setDefaultAppDataValue()
//so if you want to modify the default value, please modify the APP_CONFIG_FILE_LATEST_VERSION (in MainTask.h) meanwhile, so that will be store to the nvm file
static void setDefaultAppDataValue(void)
{
    UINT8  i = 0;
    memset(&AppNVMData, 0x00, sizeof(AppNVMDataType));
    CHAR defaultBattSN[BATT_SN_LEN] = DEFAULT_SN;
	for(i=0;i<BATT_SN_LEN;i++)
	{
		AppNVMData.battSN[i] = defaultBattSN[i];  //the defult sn is "0000..0000"(17 bytes ASCII)
	}
	AppNVMData.isBattLocked = false;
	AppNVMData.chargEndWorkTime = 30;  //30 min
	AppNVMData.wakeupWorkTime = 2;	//2min
	AppNVMData.sleepTime = 5;			//5min
    AppNVMData.HwVersion = HWVERSION;
    AppNVMData.BLSwVersion = BLSWVERSION;
    AppNVMData.DrvSwVersion = DRVSWVERSION;
    AppNVMData.AppSwVersion = APPSWVERSION;
    AppNVMData.EOLState = EOLSTATE; //未进行下线配置，北京电池测试默认进行下线测试
    return;
}

static void appSaveDataInfo(void)
{
    OSAFILE fp = PNULL;
    AppDataHeader  AppDataHr;
    UINT32  writeCount = 0;
    fp = OsaFopen(APP_DATAINFO_FILE_NAME, "wb");   //read & write
    if(OsaFseek(fp, 0, SEEK_SET) != 0)
	{
        #ifdef USING_PRINTF
				printf("Seek file failed [%d] \r\n",__LINE__);
	    #endif
	    OsaFclose(fp);
        return;
	}
    if (fp == PNULL)
    {
	    #ifdef USING_PRINTF
			printf(" NVM, can't open/create NVM: 'qxappconfig.nvm', save NVM failed\n");
		#endif
        return;
    }
    AppDataHr.fileBodySize   = sizeof(AppDataInfo); 
    AppDataHr.checkSum       = OsaCalcCrcValue((UINT8 *)&AppDataInfo, sizeof(AppDataInfo));
    #ifdef USING_PRINTF
		printf(" AppDataHr.fileBodySize:%x\n",AppDataHr.fileBodySize);
	#endif
    writeCount = OsaFwrite(&AppDataHr, sizeof(AppDataHr), 1, fp);
    if (writeCount != 1)
    {
        #ifdef USING_PRINTF
                    printf(" NVM: 'qxappconfig.nvm', write the file header failed\n");
        #endif
        OsaFclose(fp);
        return;
    }
    AppDataInfo.appDataModify = FALSE;
    writeCount = OsaFwrite(&AppDataInfo, sizeof(AppDataInfo), 1, fp);
    if (writeCount != 1)
    {
        #ifdef USING_PRINTF
			printf(" NVM: 'qxappconfig.nvm', write the file body failed\n");
		#endif
    }
    OsaFclose(fp);
    return;
}

static void LoadAppDataInfo(void)
{
    OSAFILE fp = PNULL;
    UINT32  readCount = 0;
    //UINT8   crcCheck = 0;
    AppDataHeader  AppDataHr;
    void    *pReadAppConfig = (void *)&AppDataInfo;
    fp = OsaFopen(APP_DATAINFO_FILE_NAME, "rb");   //read only
    if (fp == PNULL)
    {
        #ifdef USING_PRINTF
            printf(" NVM, can't open NVM: 'qxappData.nvm', use the defult value\n");
        #endif
        setDefaultAppDataInfo();
        appSaveDataInfo();
        return;
    }

    readCount = OsaFread(&AppDataHr, sizeof(AppDataHr), 1, fp);
    UINT8 readtimes=0;
    while (readCount != 1 && readtimes<=5 )
    {
        readtimes++;
        readCount = OsaFread(&AppDataHr, sizeof(AppDataHr), 1, fp);
        osDelay(10);
    }
    if(readtimes>5)
    {
        OsaFclose(fp);
        setDefaultAppDataInfo();
        appSaveDataInfo();
        return;
    }
    if(AppDataHr.fileBodySize==sizeof(AppDataInfo))//结构体数据没变动，可直接读出
    {
        readCount = OsaFread(pReadAppConfig, AppDataHr.fileBodySize, 1, fp);
        //crcCheck = OsaCalcCrcValue((UINT8 *)pReadAppConfig, sizeof(AppNVMData));
        readtimes = 0;
        while (readtimes<=5 && readCount != 1)
        {
            readtimes++;
            readCount = OsaFread(pReadAppConfig, AppDataHr.fileBodySize, 1, fp);
            //crcCheck = OsaCalcCrcValue((UINT8 *)pReadAppConfig, sizeof(AppNVMData));
        }
    }
    else
    {
        setDefaultAppDataInfo();
        readCount = OsaFread(pReadAppConfig, AppDataHr.fileBodySize, 1, fp);
        //crcCheck = OsaCalcCrcValue((UINT8 *)pReadAppConfig, sizeof(AppNVMData));
        readtimes = 0;
        while (readtimes<=5 && readCount != 1)
        {
            readtimes++;
            readCount = OsaFread(pReadAppConfig, AppDataHr.fileBodySize, 1, fp);
            //crcCheck = OsaCalcCrcValue((UINT8 *)pReadAppConfig, sizeof(AppNVMData));
        }
        appSaveDataInfo();
        #ifdef USING_PRINTF
            printf(" Struct update\n");
        #endif
    }
    OsaFclose(fp);
    return;
}

//无法找到文件时的第一次启动，后续不会启动此函数
static void setDefaultAppDataInfo(void)
{
    memset(&AppDataInfo, 0x00, sizeof(AppDataInfo));
    AppDataInfo.appDataModify = false;
    AppDataInfo.BmsChrgInfoSendFreq = 5;
    AppDataInfo.BmsDisChrgInfoSendFreq = 30;
    AppDataInfo.GpsChrgInfoSendFreq = 30;
    AppDataInfo.GpsDisChrgInfoSendFreq = 5;
    AppDataInfo.BattCurrentNegFlag = 1;
    AppDataInfo.CumulativeCapacity = 0;
    AppDataInfo.AccMileage = 0;
    AppDataInfo.SysReStart = 0;
    return;
}


